package de.draegerit.esptutorialapp;

import android.Manifest;
import android.app.AlertDialog;
import android.app.ProgressDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.pm.PackageManager;
import android.net.ConnectivityManager;
import android.net.NetworkInfo;
import android.net.wifi.WifiManager;
import android.os.AsyncTask;
import android.support.annotation.NonNull;
import android.support.v4.app.ActivityCompat;
import android.support.v4.content.ContextCompat;
import android.support.v7.app.AppCompatActivity;
import android.os.Bundle;
import android.util.Log;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;
import android.widget.TextView;

import com.google.gson.Gson;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.net.URL;
import java.net.URLConnection;

public class MainActivity extends AppCompatActivity {

    private EditText ipAdressEditText;

    private EditText editText;

    private Button wellcomeBtn;
    private Button fotoresistorBtn;

    private TextView textView;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);

        ipAdressEditText = findViewById(R.id.ipAdressEditText);

        editText = findViewById(R.id.editText);
        wellcomeBtn = findViewById(R.id.wellcomeBtn);
        wellcomeBtn.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                if (isNetworkAvailable()) {
                    sendWellcomeGreetingRequest(ipAdressEditText.getText().toString(), editText.getText().toString());
                }
            }
        });

        fotoresistorBtn = findViewById(R.id.fotoresistorBtn);
        fotoresistorBtn.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                if (isNetworkAvailable()) {
                    sendFotoresistorRequest(ipAdressEditText.getText().toString());
                }
            }
        });

        textView = findViewById(R.id.textView);
    }

    private boolean isNetworkAvailable() {
        ConnectivityManager connectivityManager
                = (ConnectivityManager) getSystemService(Context.CONNECTIVITY_SERVICE);
        NetworkInfo activeNetworkInfo = null;
        if (connectivityManager != null) {
            activeNetworkInfo = connectivityManager.getActiveNetworkInfo();
        }
        boolean available = activeNetworkInfo != null && activeNetworkInfo.isConnected();
        if (!available) {
            AlertDialog.Builder alertDialogBuilder = new AlertDialog.Builder(MainActivity.this);
            alertDialogBuilder.setTitle(getResources().getString(R.string.dialog_error));
            alertDialogBuilder
                    .setMessage(getResources().getString(R.string.msg_no_networkconnection))
                    .setCancelable(false)
                    .setPositiveButton(getResources().getString(R.string.ok), new DialogInterface.OnClickListener() {
                        public void onClick(DialogInterface dialog, int id) {
                            dialog.cancel();
                        }
                    })
                    .setNegativeButton(getResources().getString(R.string.activate_wifi), new DialogInterface.OnClickListener() {
                        public void onClick(DialogInterface dialog, int id) {
                            WifiManager wifi = (WifiManager) getSystemService(Context.WIFI_SERVICE);
                            wifi.setWifiEnabled(true);
                            dialog.cancel();
                        }
                    });

            AlertDialog alertDialog = alertDialogBuilder.create();
            alertDialog.show();
        }
        return available;
    }

    private void sendWellcomeGreetingRequest(String ipAdresse, String text) {
        RequestAsyncTask connectionAsyncTask = new RequestAsyncTask(ipAdresse, text, EAction.GREETING);
        connectionAsyncTask.executeOnExecutor(AsyncTask.THREAD_POOL_EXECUTOR);
    }

    private void sendFotoresistorRequest(String ipAdresse) {
        RequestAsyncTask connectionAsyncTask = new RequestAsyncTask(ipAdresse, EAction.FOTORESISTOR);
        connectionAsyncTask.executeOnExecutor(AsyncTask.THREAD_POOL_EXECUTOR);
    }

    class RequestAsyncTask extends AsyncTask<Void, Void, String> {

        private final String text;
        private String ipAdresse;

        private String errorMsg;

        private EAction action;

        public RequestAsyncTask(String ipAdresse, String text, EAction action) {
            this.ipAdresse = ipAdresse;
            this.text = text;
            this.action = action;
        }

        public RequestAsyncTask(String ipAdresse, EAction action) {
            this(ipAdresse, null, action);
        }

        @Override
        protected String doInBackground(Void... voids) {
            try {
                StringBuffer urlBuffer = new StringBuffer();
                urlBuffer.append("http://");
                urlBuffer.append(ipAdresse);
                urlBuffer.append("/");
                urlBuffer.append(action.getAdress());
                if (action.isHasParameter()) {
                    switch (action) {
                        case GREETING:
                            urlBuffer.append(text);
                            break;
                    }
                }

                URL url = new URL(urlBuffer.toString());
                URLConnection conn = url.openConnection();
                conn.setDoOutput(true);
                return readResult(conn);
            } catch (Exception e) {
                errorMsg = e.getMessage();
                e.printStackTrace();
            }
            return "";
        }

        private String readResult(URLConnection conn) throws IOException {
            BufferedReader reader = new BufferedReader(new InputStreamReader(conn.getInputStream()));
            StringBuilder sb = new StringBuilder();
            String line;
            while ((line = reader.readLine()) != null) {
                sb.append(line);
            }
            String result = sb.toString();
            Log.i("ESPTutorialApp", result);
            return result;
        }

        private void showErrorMessage(String message) {
            AlertDialog.Builder alertDialogBuilder = new AlertDialog.Builder(MainActivity.this);
            alertDialogBuilder.setTitle(getResources().getString(R.string.dialog_error));
            alertDialogBuilder
                    .setMessage(message)
                    .setCancelable(false)
                    .setPositiveButton(getResources().getString(R.string.ok), new DialogInterface.OnClickListener() {
                        public void onClick(DialogInterface dialog, int id) {
                            dialog.cancel();
                        }
                    });

            AlertDialog alertDialog = alertDialogBuilder.create();
            alertDialog.show();
        }


        @Override
        protected void onPostExecute(String result) {
            super.onPostExecute(result);
            Log.i("ESPTutorialApp", result);
            if (errorMsg == null && result.trim().length() > 0) {
                Gson gson = new Gson();
                EspMessage message = gson.fromJson(result, EspMessage.class);
                if (message != null) {
                    textView.setText(message.getMsg());
                }
            } else if (errorMsg != null) {
                showErrorMessage(errorMsg);
            }
        }


    }


}
